require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.widget.EditText"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"
import "android.media.AudioManager"
import "android.os.Handler"
import "android.os.Looper"
import "java.io.File"

-- Substitution table for replacing spoken numbers with symbols/emojis/links
local substitutionTable = {
    ["1"] = ".", ["2"] = "?", ["3"] = ",", ["4"] = "!", ["5"] = "+", ["6"] = "🫡",
    ["7"] = "🤔", ["8"] = "🇺🇿", ["9"] = "👍", ["10"] = "😄", ["11"] = "😁", 
    ["12"] = "😉", ["13"] = "👏", ["14"] = "😆", ["15"] = "😀", ["16"] = "🤣", 
    ["17"] = "😨", ["18"] = "😌", ["19"] = "😴", ["20"] = "🥱", ["21"] = "😪", 
    ["22"] = "☺️", ["23"] = "😠", ["24"] = "😌", ["25"] = "😯", 
    ["26"] = "https://t.me/Jieshuo_screenreaderUZ_chat", ["27"] = "https://t.me/Jieshuo_screenreaderUZ",
    ["28"] = "😢", ["29"] = "🍨", ["31"] = "🙂", ["32"] = "😂", ["33"] = "😝", 
 ["34"] = "🤥", ["35"] = "😊", ["36"] = "@vid ", ["37"] = "https://t.me/plagin_uz", 
}

local appStartFilePath = "/storage/emulated/0/app_start_hasanboy.txt"
local scriptToCallPath = "/storage/emulated/0/解说/Plaginlar/ovozli kiritish1.2/uz."
local appStartThreshold = 2
local maxTimeDifference = 1000 -- 1 seconds in milliseconds

-- Function to run a task on the main thread
function runOnMainThread(fn)
    Handler(Looper.getMainLooper()).post(fn)
end

-- Functions for reading and writing the app start count for multiple starts tracking
function readAppStartCount()
    local file = io.open(appStartFilePath, "r")
    if file then
        local data = file:read("*a")
        file:close()
        if data then
            local count, lastTime = data:match("(%d+)%s+(%d+)")
            return tonumber(count), tonumber(lastTime)
        end
    end
    return 0, 0
end

function writeAppStartCount(count, lastTime)
    local file = io.open(appStartFilePath, "w")
    file:write(count .. " " .. lastTime)
    file:close()
end

-- Function to call the external Lua script from the specified file path
function callExternalScript()
    local file = io.open(scriptToCallPath, "r")
    if file then
        local scriptContent = file:read("*a")
        file:close()
        assert(load(scriptContent))()
    else
        service.speak(" " .. scriptToCallPath)
    end
end

-- Initialize app start tracking to monitor rapid starts
function initializeAppStartTracking()
    local count, lastTime = readAppStartCount()
    local currentTime = os.time() * 1000
    if currentTime - lastTime < maxTimeDifference then
        count = count + 1
    else
        count = 1
    end
    lastTime = currentTime
    if count >= appStartThreshold then
        callExternalScript()
        count = 0
    end
    writeAppStartCount(count, lastTime)
end

-- Capitalize the first word of the given text
function capitalizeFirstWord(text)
    local firstChar = text:sub(1, 1)
    local restOfString = text:sub(2)
    return firstChar:upper() .. restOfString
end

-- Substitute words based on the substitution table
function substituteWords(text)
    for word, substitution in pairs(substitutionTable) do
        text = text:gsub("%f[%a]" .. word .. "%f[%A]", substitution)
    end
    return text
end

-- Check for internet connection
function hasInternetConnection()
    local connectivityManager = this.getSystemService(Context.CONNECTIVITY_SERVICE)
    local networkInfo = connectivityManager.getActiveNetworkInfo()
    return networkInfo ~= nil and networkInfo.isConnected()
end

-- Vibrate the phone if it is in silent mode
function vibrateIfSilent()
    local audioManager = this.getSystemService(Context.AUDIO_SERVICE)
    if audioManager.getRingerMode() == AudioManager.RINGER_MODE_VIBRATE then
        local vibrator = this.getSystemService(Context.VIBRATOR_SERVICE)
        if vibrator.hasVibrator() then
            vibrator.vibrate(200) -- 200 ms duration
        end
    end
end

-- Get the EditText, but allow listening even if the text field is not available
function getEditText()
    local editText = service.getEditText()
    if editText == nil then
        service.speak("")
        return nil
    else
        return editText
    end
end

-- Function to delete specific lines of text
function deleteLine(lineNumber)
    local editText = getEditText()
    local str = service.getText(editText)
    if not str or str == "" then
        service.speak("Oʻchirish uchun matn topilmadi.")
        return
    end
    local lines = {}
    for line in str:gmatch("[^\r\n]+") do
        table.insert(lines, line)
    end
    if lineNumber == 1 then
        if #lines == 0 then
            service.speak("Oʻchirish uchun matn topilmadi.")
        else
            local deletedLine = lines[1]
            table.remove(lines, 1)
            local edited_str = table.concat(lines, "\n")
            service.setText(edited_str)
            service.speak("Oʻchirildi: " .. deletedLine)
        end
    elseif lineNumber == 2 then
        if #lines == 0 then
            service.speak("O'chirish uchun matn topilmadi.")
        else
            local lastLine = lines[#lines]
            table.remove(lines, #lines)
            local edited_str = table.concat(lines, "\n")
            service.setText(edited_str)
            service.speak("Oʻchirildi: " .. lastLine)
        end
    end
end

-- Function to check if text contains numbers
function containsNumbers(text)
    return text:find("%d") ~= nil
end

-- Stop the speech recognizer
function kill()
    if speechRecognizer ~= nil then
        speechRecognizer.stopListening()
        speechRecognizer.destroy()
        speechRecognizer = nil
    end
    service.speak("Eshitish to'xtatildi.")
end

-- Start listening with speech recognizer
function startListening()
    vibrateIfSilent() -- Vibrate if the phone is in silent mode
    if not hasInternetConnection() then
        service.speak("Internet mavjud emas, iltimos tekshiring.")
        return false
    end
    local verbalizationDetected = false
    local editText = getEditText() -- Still retrieve EditText if it exists

    if speechRecognizer == nil then
        speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)
    end
speechListener = RecognitionListener{
    onResults = function(results)
        data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
        if data ~= nil and data.size() > 0 then
            verbalizationDetected = true
            local recognizedText = data.get(0)
            local isSymbol = substitutionTable[recognizedText:lower()]

            -- Check for the '50' command to take a picture
            if recognizedText == "50" then
                service.speak("50 buyrug'i qabul qilindi, surat olinmoqda.")
                dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "ovozli kiritish1.2.")
require "import"
import "android.content.Intent"
import "android.graphics.Bitmap"
import "android.net.Uri"
import "java.io.*"
import "com.androlua.*"
app_pkg="com.bemyeyes.bemyeyes"
store_url="market://details?id="
screenshot_dir = "/sdcard/ovozli kiritish1.2/"
screenshot_path = screenshot_dir .. "screenshot.png"
-- Ensure the "ovozli kiritish1.2" directory exists
File(screenshot_dir).mkdirs()
task(500, function()
  if this.getPackageManager().getLaunchIntentForPackage(app_pkg) then
    this.getScreenShot({
      onScreenCaptureDone = function(A0_4)
        A0_4.compress(Bitmap.CompressFormat.PNG, 90, FileOutputStream(File(screenshot_path)))
        this.startActivity(Intent(Intent.ACTION_SEND).setType("image/*").setPackage(app_pkg).putExtra(Intent.EXTRA_STREAM, this.getUriForFile(File(screenshot_path))))
        
        -- Schedule deletion of the screenshot after 5 seconds
        task(5000, function()
          File(screenshot_path).delete()
        end)
      end
    })
  else
    this.startActivity(Intent(Intent.ACTION_VIEW).setData(Uri.parse(store_url .. app_pkg)))
  end
end)

            -- Check for the '51' command to execute the specific action
            elseif recognizedText == "51" then
                service.speak("51 buyrug'i qabul qilindi") 
dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "ovozli kiritish1.2.")
require "import"
import "android.content.Intent"
import "android.graphics.Bitmap"
import "android.net.Uri"
import "java.io.*"
import "com.androlua.*"
app_pkg="kr.tuat.sullivanfinder"
store_url="market://details?id="
screenshot_dir = "/sdcard/ovozli kiritish1.2/"
screenshot_path = screenshot_dir .. "screenshot.png"
-- Ensure the "ovozli kiritish1.2" directory exists
File(screenshot_dir).mkdirs()
task(500, function()
  if this.getPackageManager().getLaunchIntentForPackage(app_pkg) then
    this.getScreenShot({
      onScreenCaptureDone = function(A0_4)
        A0_4.compress(Bitmap.CompressFormat.PNG, 90, FileOutputStream(File(screenshot_path)))
        this.startActivity(Intent(Intent.ACTION_SEND).setType("image/*").setPackage(app_pkg).putExtra(Intent.EXTRA_STREAM, this.getUriForFile(File(screenshot_path))))
        
        -- Schedule deletion of the screenshot after 5 seconds
        task(5000, function()
          File(screenshot_path).delete()
        end)
      end
    })
  else
    this.startActivity(Intent(Intent.ACTION_VIEW).setData(Uri.parse(store_url .. app_pkg)))
  end
end)
 
-- Check for the '52' command to execute the specific action
            elseif recognizedText == "52" then
                service.speak("52 buyrug'i qabul qilindi") dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "ovozli kiritish1.2.")
require "import"
import "android.content.Intent"
import "android.graphics.Bitmap"
import "android.net.Uri"
import "java.io.*"
import "com.androlua.*"
app_pkg="com.sparklingapps.piccybot"
store_url="market://details?id="
screenshot_dir = "/sdcard/ovozli kiritish1.2/"
screenshot_path = screenshot_dir .. "screenshot.png"
-- Ensure the "ovozli kiritish1.2" directory exists
File(screenshot_dir).mkdirs()
task(500, function()
  if this.getPackageManager().getLaunchIntentForPackage(app_pkg) then
    this.getScreenShot({
      onScreenCaptureDone = function(A0_4)
        A0_4.compress(Bitmap.CompressFormat.PNG, 90, FileOutputStream(File(screenshot_path)))
        this.startActivity(Intent(Intent.ACTION_SEND).setType("image/*").setPackage(app_pkg).putExtra(Intent.EXTRA_STREAM, this.getUriForFile(File(screenshot_path))))
        
        -- Schedule deletion of the screenshot after 5 seconds
        task(5000, function()
          File(screenshot_path).delete()
        end)
      end
    })
  else
    this.startActivity(Intent(Intent.ACTION_VIEW).setData(Uri.parse(store_url .. app_pkg)))
  end
end)
 -- Check for the '53' command to execute the specific action
            elseif recognizedText == "53" then
                service.speak("53 buyrug'i qabul qilindi")
require "import"
import "com.androlua.*"
import "java.util.Locale"
import "android.net.Uri"
import "android.text.Html"

-- Kiril alifbosiga o'giruvchi funktsiya
local function transliterateToCyrillic(text)
    -- Lotin alifbosidan Kirilga o'girish jadvali
    local latinToCyrillic = {
        ["sh"] = "ш", ["Sh"] = "Ш", ["ch"] = "ч", ["Ch"] = "Ч", ["yu"] = "ю", ["Yu"] = "Ю",
        ["ya"] = "я", ["Ya"] = "Я", ["g'"] = "ғ", ["G'"] = "Ғ", ["o'"] = "ў", ["O'"] = "Ў",
        ["a"] = "а", ["A"] = "А", ["b"] = "б", ["B"] = "Б", ["v"] = "в", ["V"] = "В",
        ["g"] = "г", ["G"] = "Г", ["d"] = "д", ["D"] = "Д", ["e"] = "е", ["E"] = "Е",
        ["zh"] = "ж", ["Zh"] = "Ж", ["z"] = "з", ["Z"] = "З", ["i"] = "и", ["I"] = "И",
        ["y"] = "й", ["Y"] = "Й", ["k"] = "к", ["K"] = "К", ["l"] = "л", ["L"] = "Л",
        ["m"] = "м", ["M"] = "М", ["n"] = "н", ["N"] = "Н", ["o"] = "о", ["O"] = "О",
        ["p"] = "п", ["P"] = "П", ["r"] = "р", ["R"] = "Р", ["s"] = "с", ["S"] = "С",
        ["t"] = "т", ["T"] = "Т", ["u"] = "у", ["U"] = "У", ["f"] = "ф", ["F"] = "Ф",
        ["h"] = "х", ["H"] = "Х", ["ts"] = "ц", ["Ts"] = "Ц", ["q"] = "қ", ["Q"] = "Қ",
        ["x"] = "ҳ", ["X"] = "Ҳ", ["'"] = "ъ", ["e"] = "э", ["E"] = "Э"
    }

    -- Murakkab birikmalarni avval o'girib olish
    local patterns = {"sh", "Sh", "ch", "Ch", "yu", "Yu", "ya", "Ya", "g'", "G'", "o'", "O'"}

    for _, pattern in ipairs(patterns) do
        text = text:gsub(pattern, latinToCyrillic[pattern] or pattern)
    end

    -- Qolgan bir harfli belgilarni o'zgartirish
    text = text:gsub(".", function(char)
        return latinToCyrillic[char] or char
    end)

    return text
end

-- Asosiy funksiyalar
local function translateAndDisplay()
    local textToTranslate = this.getText(node) -- Matn olish
    local cyrillicText = transliterateToCyrillic(textToTranslate) -- Transliteratsiya qilish
    service.setText(cyrillicText) -- Natijani chiqarish
    service.speak(" " .. cyrillicText) -- Natijani ovoz bilan e'lon qilish
 end   

translateAndDisplay()
require "import"
import "android.content.Context"
import "android.os.Vibrator"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(200) -- 200 millisekund davomida tebranish

-- Check for the '54' command to execute the specific action
            elseif recognizedText == "54" then
                service.speak("54 buyrug'i qabul qilindi")
require "import"
import "com.androlua.*"
import "java.util.Locale"
import "android.net.Uri"
import "android.text.Html"

-- Kiril alifbosiga o'giruvchi funktsiya
local function transliterateToLatin(text)
    -- Kiril alifbosidan Lotin alifbosiga o'girish jadvali
    local cyrillicToLatin = {
        ["ш"] = "sh", ["Ш"] = "Sh", ["ч"] = "ch", ["Ч"] = "Ch", ["ю"] = "yu", ["Ю"] = "Yu",
        ["я"] = "ya", ["Я"] = "Ya", ["ғ"] = "g'", ["Ғ"] = "G'", ["ў"] = "o'", ["Ў"] = "O'",
        ["а"] = "a", ["А"] = "A", ["б"] = "b", ["Б"] = "B", ["в"] = "v", ["В"] = "V",
        ["г"] = "g", ["Г"] = "G", ["д"] = "d", ["Д"] = "D", ["е"] = "e", ["Е"] = "E",
        ["ж"] = "zh", ["Ж"] = "Zh", ["з"] = "z", ["З"] = "Z", ["и"] = "i", ["И"] = "I",
        ["й"] = "y", ["Й"] = "Y", ["к"] = "k", ["К"] = "K", ["л"] = "l", ["Л"] = "L",
        ["м"] = "m", ["М"] = "M", ["н"] = "n", ["Н"] = "N", ["о"] = "o", ["О"] = "O",
        ["п"] = "p", ["П"] = "P", ["р"] = "r", ["Р"] = "R", ["с"] = "s", ["С"] = "S",
        ["т"] = "t", ["Т"] = "T", ["у"] = "u", ["У"] = "U", ["ф"] = "f", ["Ф"] = "F",
        ["х"] = "h", ["Х"] = "H", ["ц"] = "ts", ["Ц"] = "Ts", ["қ"] = "q", ["Қ"] = "Q",
        ["ҳ"] = "x", ["Ҳ"] = "X", ["ъ"] = "'", ["э"] = "e", ["Э"] = "E"
    }

    -- Murakkab birikmalarni avval o'girib olish
    local patterns = {"ш", "Ш", "ч", "Ч", "ю", "Ю", "я", "Я", "ғ", "Ғ", "ў", "Ў"}

    for _, pattern in ipairs(patterns) do
        text = text:gsub(pattern, cyrillicToLatin[pattern] or pattern)
    end

    -- Qolgan harflarni o'zgartirish
    text = text:gsub(".", function(char)
        return cyrillicToLatin[char] or char
    end)

    return text
end

-- Asosiy funksiyalar
local function translateAndDisplay()
    local textToTranslate = this.getText(node) -- Matn olish
    local latinText = transliterateToLatin(textToTranslate) -- Transliteratsiya qilish
    service.setText(latinText) -- Natijani chiqarish
    service.speak(" " .. latinText) -- Natijani ovoz bilan e'lon qilish
    end

translateAndDisplay()
require "import"
import "android.content.Context"
import "android.os.Vibrator"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(200) -- 200 millisekund davomida tebranish
-- Check for the '55' command to execute the specific action
            elseif recognizedText == "55" then
                service.speak("")
                if service.click({{"Ovozli xabar yozish$1",
"%暂停主语音合成",
"%模拟上滑",
"[Bir marta ijro qilishni faollashtirish]",
"<Yuborish",
"%暂停主语音合成"
}}) then
                    return true
                end


-- Check for the '56' command to execute the specific action
            elseif recognizedText == "56" then
                service.speak("56 buyrug'i qabul qilindi")
require "import"
import "com.androlua.*"
import "java.util.Locale"
import "android.net.Uri"
import "android.text.Html"

local translationResult = ""

url = "https://translate.google.com/m?sl=auto&tl=en&hl="..Locale.getDefault().getLanguage().."&q="..Uri.encode(this.getText(node))

Http.get(url, function(status, result)
    if status == 200 then
        local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
        translationResult = translationResult .. translation
        service.setText(translationResult)

service.speak(" ".. translationResult)

        
    end
end)
require "import"
import "android.content.Context"
import "android.os.Vibrator"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(200) -- 200 millisekund davomida tebranish 
-- Check for the '57' command to execute the specific action
            elseif recognizedText == "57" then
                service.speak("57 buyrug'i qabul qilindi")
require "import"
import "com.androlua.*"
import "java.util.Locale"
import "android.net.Uri"
import "android.text.Html"

local translationResult = ""

url = "https://translate.google.com/m?sl=auto&tl=ru&hl="..Locale.getDefault().getLanguage().."&q="..Uri.encode(this.getText(node))

Http.get(url, function(status, result)
    if status == 200 then
        local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
        translationResult = translationResult .. translation
        service.setText(translationResult)

service.speak(" ".. translationResult)

        
    end
end)
require "import"
import "android.content.Context"
import "android.os.Vibrator"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(200) -- 200 millisekund davomida tebranish
 -- Check for the '58' command to execute the specific action
            elseif recognizedText == "58" then
                service.speak("58 buyrug'i qabul qilindi")
require "import"
import "com.androlua.*"
import "java.util.Locale"
import "android.net.Uri"
import "android.text.Html"

local translationResult = ""

url = "https://translate.google.com/m?sl=auto&tl=uz&hl="..Locale.getDefault().getLanguage().."&q="..Uri.encode(this.getText(node))

Http.get(url, function(status, result)
    if status == 200 then
        local translation = tostring(Html.fromHtml(result:match('<div class="result%-container">([^<]+)</div>')))
        translationResult = translationResult .. translation
        service.setText(translationResult)

service.speak(" ".. translationResult)

        
    end
end)
require "import"
import "android.content.Context"
import "android.os.Vibrator"

local context = activity or service
local vibrator = context.getSystemService(Context.VIBRATOR_SERVICE)
vibrator.vibrate(200) -- 200 millisekund davomida tebranish
  
-- Check for the '59' command to execute the specific action
            elseif recognizedText == "59" then
                service.speak("59 buyrug'i qabul qilindi")
if service.click({
{"%清空"
}
})
return true
end
dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "Shitirlash.")

-- Check for the '62' command to execute the specific action
            elseif recognizedText == "62" then
                service.speak("")
 require "import"
import "android.os.Environment"
import "java.io.File"
import "java.io.FileWriter"
import "java.io.BufferedWriter"

-- Fayl nomini doimiy qilib belgilash
local function getFileName(directory)
    return string.format("%s/Hujjat.txt", directory)  -- Fayl nomi Hujjat, faqat bitta faylga yoziladi
end

-- Faylni saqlash yoki davomiga yozish funksiyasi .txt formatda
local function saveToFileAsText(text)
    -- Fayl saqlanadigan papkani olish
    local directory = Environment.getExternalStorageDirectory().getAbsolutePath() .. "/Hujjatlarim"
    local dirFile = File(directory)
    if not dirFile.exists() then
        dirFile.mkdirs()  -- Agar papka mavjud bo'lmasa, uni yaratish
    end

    -- Fayl nomini olish (Hujjat nom)
    local fileName = getFileName(directory)
    local file = File(fileName)

    -- Faylga davomiga yozish
    local fileWriter = FileWriter(file, true)  -- true parametri bilan faylga yozish davom etadi
    local bufferedWriter = BufferedWriter(fileWriter)

    -- Matnni yozish
    bufferedWriter.write(text .. "\n")  -- Har bir yozuvdan keyin yangi qatordan yoziladi

    -- Yozishni tugatish va faylni yopish
    bufferedWriter.close()
    fileWriter.close()
end

-- Tanlangan matnni olish va .txt formatda saqlash funksiyasi
local function saveSelectedTextAsText()
    local textToSave = this.getText(node)  -- Matnni olish
    saveToFileAsText(textToSave)  -- Matnni faylga yozish
end

saveSelectedTextAsText()
dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "Fayl.")
-- Check for the '63' command to execute the specific action
            elseif recognizedText == "63" then
                service.speak("")

require "import"
import "android.graphics.pdf.PdfDocument"
import "android.graphics.Paint"
import "android.graphics.Canvas"
import "android.graphics.Color"
import "android.graphics.Typeface"
import "java.io.FileOutputStream"
import "android.os.Environment"
import "java.io.File"

-- Matnni sahifa kengligiga mos ravishda ajratish funksiyasi
local function splitTextToLines(text, maxWidth, paint)
    local result = {}
    local currentLine = ""
    for word in text:gmatch("%S+") do
        local testLine = currentLine == "" and word or currentLine .. " " .. word
        local lineWidth = paint.measureText(testLine)
        if lineWidth > maxWidth then
            table.insert(result, currentLine)
            currentLine = word
        else
            currentLine = testLine
        end
    end
    if currentLine ~= "" then
        table.insert(result, currentLine)
    end
    return result
end

-- Fayl nomini joriy vaqtga mos ravishda yaratish
local function getFileName(directory)
    local date = Date()
    local weekDays = {"Yakshanba", "Dushanba", "Seshanba", "Chorshanba", "Payshanba", "Juma", "Shanba"}
    local months = {"Yanvar", "Fevral", "Mart", "Aprel", "May", "Iyun", "Iyul", "Avgust", "Sentabr", "Oktabr", "Noyabr", "Dekabr"}
    local year = date.getYear() + 1900
    local month = date.getMonth() + 1
    local hour = date.getHours()
    local minute = date.getMinutes()
    local second = date.getSeconds()

    return string.format("%s/Hujjat_%02d_%02d_%02d_%s_%s_%04d.pdf", 
                         directory, hour, minute, second, weekDays[date.getDay() + 1], months[month], year)
end

-- PDF formatda faylga saqlash funksiyasi
local function saveToFileAsPDF(text)
    local pdfDocument = PdfDocument()
    local pageInfo = PdfDocument.PageInfo.Builder(595, 842, 1).create()
    local page = pdfDocument.startPage(pageInfo)
    local canvas = page.getCanvas()

    -- Matnni yozish uchun paint obyekti
    local paint = Paint()
    paint.setTextSize(14)  -- O'qish uchun qulay shrift hajmi
    paint.setColor(Color.DKGRAY)  -- Ko'z uchun qulay quyuq kulrang
    paint.setTypeface(Typeface.create("sans-serif", Typeface.NORMAL))  -- Sans-serif oddiy shrift
    paint.setFakeBoldText(false)  -- Qalin shriftni o'chirish
    paint.setTextSkewX(0)  -- Italik shriftni o'chirish

    -- Matnni qatorlarga ajratish
    local maxWidth = 555  -- Sahifa kengligi bo'yicha max uzunlikni aniqlash (chetlarni kengroq qilish uchun)
    local lines = {}
    for line in text:gmatch("[^\r\n]+") do
        local splitLines = splitTextToLines(line, maxWidth, paint)
        for _, splitLine in ipairs(splitLines) do
            table.insert(lines, splitLine)
        end
    end

    -- Matnni PDF sahifasiga joylashtirish
    local y = 40  -- Matnning boshlang'ich y pozitsiyasi
    for i, line in ipairs(lines) do
        canvas.drawText(line, 30, y, paint)  -- Chap chetdan 30 piksel ichkariga joylash
        y = y + 25  -- Qatorlar orasidagi masofa

        if y > 820 then  -- Sahifa chekkasiga yetganda yangi sahifa ochish
            pdfDocument.finishPage(page)
            page = pdfDocument.startPage(pageInfo)
            canvas = page.getCanvas()
            y = 40  -- Yangi sahifaning boshlang'ich nuqtasi
        end
    end

    pdfDocument.finishPage(page)

    -- Faylni saqlash
    local directory = Environment.getExternalStorageDirectory().getAbsolutePath() .. "/Hujjatlarim"
    local dirFile = File(directory)
    if not dirFile.exists() then
        dirFile.mkdirs()
    end

    local fileName = getFileName(directory)
    local fileOutputStream = FileOutputStream(fileName)
    pdfDocument.writeTo(fileOutputStream)
    pdfDocument.close()
    fileOutputStream.close()

end

-- Tanlangan matnni olish va PDF formatda saqlash funksiyasi
local function saveSelectedTextAsPDF()
    local textToSave = this.getText(node)
    saveToFileAsPDF(textToSave)
end

saveSelectedTextAsPDF()
dir = package.searchpath("main", package.path):match("(.*)main.lua")
require "import"
import "android.media.MediaPlayer"
function playMusic(player)
    local mp = MediaPlayer()
    mp.setOnCompletionListener(MediaPlayer.OnCompletionListener {
        onCompletion = function(mp)
            mp.release()
        end
    })
    mp.setDataSource(player)
    mp.prepare()
    mp.start()
end
playMusic(dir .. "Fayl.")
-- Check for the '64' command to execute the specific action
            elseif recognizedText == "64" then
                service.speak("")

require "import"
import "android.content.Intent"
import "android.content.pm.PackageManager"
import "android.net.Uri"
import "java.io.File"
function openCSRFolder()
    local folderPath = "/storage/emulated/0/Hujjatlarim"
    local folderUri = Uri.parse(folderPath)
    -- Crear un intento para abrir el directorio
    local intent = Intent(Intent.ACTION_VIEW)
    intent.setDataAndType(folderUri, "resource/folder")
    intent.setFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
    if intent.resolveActivity(service.getPackageManager()) then
        service.startActivity(intent)
    else
service.asyncSpeak("No application found to open the folder.")
    end
end
openCSRFolder()
return true


-- Check for the '65' command to execute the specific action
            elseif recognizedText == "65" then
                service.speak("")

-- Import necessary AndroLua classes for Android interaction
local luajava = require("luajava")
local Context = luajava.bindClass("android.content.Context")
local SpeechRecognizer = luajava.bindClass("android.speech.SpeechRecognizer")
local Intent = luajava.bindClass("android.content.Intent")
local RecognizerIntent = luajava.bindClass("android.speech.RecognizerIntent")
local Activity = luajava.bindClass("android.app.Activity")

local activity = this

-- Function to start voice recognition with uzbek language
function startVoiceRecognition(activity)

    local recognizer = SpeechRecognizer.createSpeechRecognizer(activity)
    local intent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    
    -- Set the language to uzbek (
    intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "uz")

    local recognitionListener = luajava.createProxy("android.speech.RecognitionListener", {
        onReadyForSpeech = function(params)
            -- Handle the ready for speech event
        end,
        onResults = function(params)
            local results = params.getStringArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            -- Handle the recognized speech results
            -- You can display or use the recognized text here
url= 'https://m.youtube.com/results?sp=mAEA&search_query='..results.get(0)
require 'import'
import 'android.content.Intent'
import 'android.net.Uri'
import 'java.*'

i= Intent(Intent.ACTION_VIEW)
i.setData(Uri.parse(url))
this.startActivity(Intent.createChooser(i, ''))

return true
        end,
        -- Handle other recognition events here
    })

    recognizer.setRecognitionListener(recognitionListener)
    recognizer.startListening(intent)
end

    startVoiceRecognition(activity)

-- Called when the activity is created
function onCreate(savedInstanceState)
    activity.setTitle("Voice Input Example")
end


-- Check for the '39' command to execute the specific action
            elseif recognizedText == "39" then
                service.speak("")
if service.click({
{"%链接"
}
})
return true
end
 -- Check for the '38' command to execute the specific action
            elseif recognizedText == "38" then
                service.speak("")
local fz=service.getLastSpeakText()
 service.copy(fz) 
service.speak("Nusxa olimdi."..
fz) return true
  
-- Check for the '0' command to execute the specific action
            elseif recognizedText == "0" then
                service.speak("") 
local matn = {
    "0 Yordam",
    "1. Nuqta.",
    "2. So'roq belgisi.",
    "3. Vergul.",
    "4. Undov belgisi.",
    "5. Qo'shuv belgisi.",
    "6. Salom berdim (🫡)",
    "7. O'ylab ko'rish (🤔)",
    "8. O'zbekiston bayrog'i (🇺🇿)",
    "9. Ajoyib! (👍)",
    "10. Kulib turish (😄)",
    "11. Baxtli (😁)",
    "12. Ko'z qisish (😉)",
    "13. Qarsak (👏)",
    "14. Kulgi (😆)",
    "15. Kulganda (😀)",
    "16. Katta kulgi (🤣)",
    "17. Hayratlangan (😨)",
    "18. Tinchlikda (😌)",
    "19. Uxlash (😴)",
    "20. Charchagan (🥱)",
    "21. Charchagan (😪)",
    "22. Tabassum (☺️)",
    "23. Jahli chiqdi (😠)",
    "24. Tinchlikda (😌)",
    "25. Ajablangan (😯)",
    "26. https://t.me/Jieshuo_screenreaderUZ_chat",
    "27. https://t.me/Jieshuo_screenreaderUZ",
    "28. G'amgin (😢)",
    "29. Muzqaymoq (🍨)",
    "31. Oddiy tabassum (🙂)",
    "32. Kulgi (😂)",
    "33. Til chiqardi (😝)",
    "34. Aldash (🤥)",
    "35. Baxtli (😊)",
    "36. @vid",
    "37. https://t.me/plagin_uz",
    "38. Oxirgi aytilgan jumlani nusxalash",
    "39. Havola",
    "50. Be My Eyes-ga suratingizni ulashing",
    "51. Sullivan Finder-ga suratingizni ulashing",
    "52. PiccyBot-ga suratingizni ulashing",
    "53. Lotin alifbosida yozilgan matningizni kirill alifbosiga aylantiring",
    "54. Kirill alifbosida yozilgan matningizni lotin alifbosiga aylantiring",
    "55. Bir marotaba tinglangach, o'chib ketadigan ovozli xabar jo'nating",
    "56. Istalgan matnni ingliz tiliga tarjima qiling",
    "57. Istalgan matningizni rus tiliga tarjima qiling",
    "58. Istalgan matningizni o'zbek tiliga tarjima qiling",
    "59. Matn maydonini tozalang",
    "60. Birinchi yozilgan qatorni o'chiring",
    "61. Oxirgi yozilgan qatorni o'chiring",
    "62. Istalgan matningizni txt formatda telefoningiz xotirasiga saqlang",
    "63. Istalgan matningizni pdf formatda telefoningiz xotirasiga saqlang",
    "64. Saqlagan hujjatlaringizni ko'rish",
    "65. YouTubedan ovozli izlang"
}

-- Har bir qatorni alohida chop etish
for _, qator in ipairs(matn) do
    print(qator)
end
   elseif isSymbol then
                runOnMainThread(function()
                    service.insertText(editText, isSymbol .. "\n")
                end)
                vibrateIfSilent() -- Vibrate when a special symbol is inserted

            elseif recognizedText == "60" then
                deleteLine(1)

            elseif recognizedText == "61" then
                deleteLine(2)

            elseif not containsNumbers(recognizedText) then
                recognizedText = capitalizeFirstWord(recognizedText)
                recognizedText = substituteWords(recognizedText)
                runOnMainThread(function()
                    service.insertText(editText, recognizedText)
                end)
                service.speak(recognizedText)

            else
                service.speak("")
            end
        end
        if not verbalizationDetected then
            kill()
            return false
        end
    end,
    onError = function(errorCode)
        if errorCode == SpeechRecognizer.ERROR_AUDIO or errorCode == SpeechRecognizer.ERROR_NO_MATCH then
            vibrateIfSilent() -- Vibrate if the microphone is off
        end
        kill()
        return false
    end
}
    recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "uz_UZ")
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
    speechRecognizer.startListening(recognizerIntent)
    speechRecognizer.setRecognitionListener(speechListener)
    
    return true
end

initializeAppStartTracking()
startListening()
return true